#' Enrichment of known oncogenic pathways
#' @description Checks for enrichment of known oncogenic pathways
#' @references Sanchez-Vega F, Mina M, Armenia J, Chatila WK, Luna A, La KC, Dimitriadoy S, Liu DL, Kantheti HS, Saghafinia S et al. 2018. Oncogenic Signaling Pathways in The Cancer Genome Atlas. Cell 173: 321-337 e310
#'
#' @details
#' Oncogenic signalling pathways are derived from TCGA cohorts. See reference for details.
#' @param maf an \code{\link{MAF}} object generated by \code{\link{read.maf}}
#' @return Prints fraction of altered pathway
#' @seealso \code{\link{PlotOncogenicPathways}}
#' @export
#' @examples
#' laml.maf <- system.file("extdata", "tcga_laml.maf.gz", package = "maftools")
#' laml <- read.maf(maf = laml.maf)
#' OncogenicPathways(maf = laml)

OncogenicPathways = function(maf){
  pathdb <- system.file("extdata", "oncogenic_sig_patwhays.tsv", package = "maftools")
  pathdb = data.table::fread(input = pathdb)
  pathdb_size = pathdb[,.N,Pathway]
  pathdb = split(pathdb, as.factor(pathdb$Pathway))

  altered_pws = lapply(pathdb, function(pw){
                  x = suppressMessages(try(genesToBarcodes(maf = maf, genes = pw$Gene)))
                  if(class(x) == "try-error"){
                    pw_genes = NULL
                  }else{
                    pw_genes = suppressMessages(names(genesToBarcodes(maf = maf, genes = pw$Gene, justNames = TRUE)))
                  }
                  pw_genes
                })

  altered_pws = as.data.frame(t(data.frame(lapply(altered_pws, length))))
  data.table::setDT(x = altered_pws, keep.rownames = TRUE)
  colnames(altered_pws) = c("Pathway", "n_affected_genes")
  altered_pws$Pathway = gsub(pattern = "\\.", replacement = "-", x = altered_pws$Pathway)
  altered_pws = merge(pathdb_size, altered_pws, all.x = TRUE)

  altered_pws[, fraction_affected := n_affected_genes/N]
  altered_pws = altered_pws[order(n_affected_genes, fraction_affected, decreasing = TRUE)]

  message("Pathway alteration fractions")
  print(altered_pws)

  altered_pws = altered_pws[!n_affected_genes %in% 0]

  if(nrow(altered_pws) > 0){
    par(mar = c(4, 3, 2, 2))
    bubble_plot(plot_dat = altered_pws, lab_dat = altered_pws, y_var = "n_affected_genes",
                x_var = "N", text_var = "Pathway", bubble_var = "fraction_affected", bubble_size = 5)
    mtext(text =  "Pathway size", side = 1, line = 2)
    mtext(text =  "# affected genes", side = 2, line = 2)
  }
}
